//+------------------------------------------------------------------+
//|                                          EA_1477022Truecross.mq4 |
//|                                         Copyright  2010, MatSol |
//|                                            matunga.sol@gmail.com |
//|                                                      Version 1.3 |
//+------------------------------------------------------------------+
#property copyright "Copyright  2010, MatSol"
#property link      "matunga.sol@gmail.com"

#include <stderror.mqh>
#include <stdlib.mqh>

/////////////////////////////////////////////////////////////////////
// Extern Inputs
/////////////////////////////////////////////////////////////////////
extern int         EMA_SHORTER_PERIOD = 5;
extern int         EMA_LONGER_PERIOD  = 13;
extern int         STATUS_WINDOW_LOCATION      = 2;             // TL=1, TR=2, BR=3, BL=4

extern bool UseTrailingStop = True;
extern int TrailingStop = 5;

extern bool AutoSL= True;
extern int SL_Pips= 30;

/////////////////////////////////////////////////////////////////////
// Local variables
/////////////////////////////////////////////////////////////////////
int         SLIPPAGE = 10;
int         DIGIT_MULTIPLIER = 1;

int         m_nOrdersTotal = -1;
int         m_nOrdersHistoryTotal = -1;
double      m_dAccountBalance = -1.0;
double      m_dAccountMargin = -1.0;

int         m_nInitLnPosCount = 0;
int         m_nInitLnArray[16];
double      m_dInitLnTriggerLevels[16];

int         m_nInitShPosCount = 0;
int         m_nInitShArray[16];
double      m_dInitShTriggerLevels[16];

int         m_nPartPosCount = 0;
int         m_nPartArray[16];
int         m_nPartPeriod[16];

datetime    m_dtTickTime = 0;
datetime    m_dtBarTime = 0;
bool        m_bIsDirty = false;
string      m_sConfigFile = "";
bool        m_bIsEaEnabled = false;
string      TEXT_ID_LINE_1 = "tx_1477022_1";    // Text Line displayed on the EA chart.
string      TEXT_ID_LINE_2 = "tx_1477022_2";    // Text Line displayed on the EA chart.


/////////////////////////////////////////////////////////////////////
// init
/////////////////////////////////////////////////////////////////////
int init()
{
   m_nOrdersTotal = -1;
   m_nOrdersHistoryTotal = -1;
   m_dAccountBalance = -1.0;
   m_dAccountMargin = -1.0;
   
   DIGIT_MULTIPLIER = 1;
   if( Digits == 3 || Digits == 5 ) DIGIT_MULTIPLIER = 10;
   m_sConfigFile = "1477022\\" + AccountNumber() + "_" + Symbol() + ".cfg";

   TrailingStop = TrailingStop * DIGIT_MULTIPLIER ;
   SL_Pips = SL_Pips * DIGIT_MULTIPLIER ;
   
   
   m_bIsEaEnabled = false;
   if( EMA_SHORTER_PERIOD < 3 )
   {
      Alert(Symbol() + " : The value of EMA_SHORTER_PERIOD should be greater than or equal to 3.");
      return;
   }
   if( EMA_LONGER_PERIOD < 3 )
   {
      Alert(Symbol() + " : The value of EMA_LONGER_PERIOD should be greater than or equal to 3.");
      return;
   }
   if( EMA_SHORTER_PERIOD == EMA_LONGER_PERIOD )
   {
      Alert(Symbol() + " : The value of EMA_LONGER_PERIOD and EMA_SHORTER_PERIOD cannot be identical .");
      return;
   }
   
   if( EMA_SHORTER_PERIOD > EMA_LONGER_PERIOD )
      swapInt(EMA_SHORTER_PERIOD, EMA_LONGER_PERIOD);
      
   m_bIsEaEnabled = true;
   return(0);
}


/////////////////////////////////////////////////////////////////////
// deinit
/////////////////////////////////////////////////////////////////////
int deinit()
{
   ObjectDelete(TEXT_ID_LINE_1);
   ObjectDelete(TEXT_ID_LINE_2);
   return(0);
}

/////////////////////////////////////////////////////////////////////
// start
/////////////////////////////////////////////////////////////////////
int start()
{
  
   //+-------Auto Stop Loss-------+
   for( int a = 0; a < OrdersTotal() ; a++ ){
      if( OrderSelect(a, SELECT_BY_POS) ) {            
         if (OrderStopLoss()==0 && AutoSL && SL_Pips>0 ){
            if (OrderType() == OP_BUY  ){
               OrderModify(OrderTicket(), OrderOpenPrice(), Bid - Point * SL_Pips, OrderTakeProfit(), 0, MediumSeaGreen);   
            }
            else if (OrderType() == OP_SELL ){
               OrderModify(OrderTicket(), OrderOpenPrice(), Ask + Point * SL_Pips, OrderTakeProfit(), 0, DarkOrange);   
            }
         }  
      }
   }
   //+----------------------------+
   
   if( !m_bIsEaEnabled ) return;
   if( IsPositionStateChanged() )
      IdentifyPositions();
   
   m_bIsDirty = false;
   
   if( m_nInitLnPosCount > 0 )
      ValidateInitPositions(m_nInitLnPosCount, m_nInitLnArray, m_dInitLnTriggerLevels, Bid, true);
     
   if( m_nInitShPosCount > 0 )
      ValidateInitPositions(m_nInitShPosCount, m_nInitShArray, m_dInitShTriggerLevels, Ask, false);
     
   if( m_bIsDirty )
      IdentifyPositions();
      
   if( m_nPartPosCount > 0 && IsNewBar() )
      ValidatePartialPositions();
   
   
   return(0);
}

/////////////////////////////////////////////////////////////////////
// IdentifyPositions
/////////////////////////////////////////////////////////////////////
void IdentifyPositions()
{
   m_nInitLnPosCount = 0;
   m_nInitShPosCount = 0;
   m_nPartPosCount = 0;
   for( int i = 0; i < OrdersTotal() ; i++ )
   {
      if( OrderSelect(i, SELECT_BY_POS) )
      {
         if( OrderSymbol() != Symbol() ) continue;
         if( OrderType() == OP_BUY || OrderType() == OP_SELL )
         {
            if( StringFind(OrderComment(), "#") >= 0 )
            {
               //+----------Trailing Stop -------------+
               if(OrderType() == OP_BUY) {
                  if(UseTrailingStop && TrailingStop > 0) {                 
                     if(Bid - OrderOpenPrice() > Point * TrailingStop) {
                        if(OrderStopLoss() < Bid - Point * TrailingStop) {
                           OrderModify(OrderTicket(), OrderOpenPrice(), Bid - Point * TrailingStop, OrderTakeProfit(), 0, MediumSeaGreen);
                           continue;
                        }
                     }
                  }               
               }else if(OrderType() == OP_SELL) {
                  if(UseTrailingStop && TrailingStop > 0) {                 
                     if((OrderOpenPrice() - Ask) > (Point * TrailingStop)) {
                        if((OrderStopLoss() > (Ask + Point * TrailingStop)) || (OrderStopLoss() == 0)) {
                           OrderModify(OrderTicket(), OrderOpenPrice(), Ask + Point * TrailingStop, OrderTakeProfit(), 0, DarkOrange);
                           continue;
                        }
                     }
                  }              
               }
               //+-----------------------------------------+
                 
               int nPeriod = GetPeriodFromComment(OrderComment());
               if( nPeriod >= PERIOD_M1 )
               {
                  m_nPartArray[m_nPartPosCount] = GetPartialOrderTicket(OrderComment());
                  m_nPartPeriod[m_nPartPosCount] = nPeriod;                          
                  m_nPartPosCount++;
                  
               }
            }
            else if (OrderStopLoss() > 0.0 && GetPeriodEnumerator(OrderComment()) >=  PERIOD_M1)
            {
               int nLastError = 0;
               SaveProperty(m_sConfigFile, OrderTicket(), GetPeriodEnumerator(OrderComment()), nLastError);
               if( OrderType() == OP_BUY ) 
               {
                  m_nInitLnArray[m_nInitLnPosCount] = OrderTicket();
                  m_dInitLnTriggerLevels[m_nInitLnPosCount] = OrderOpenPrice() + (OrderOpenPrice() - OrderStopLoss())/2.0;
                  m_nInitLnPosCount++;
               }
               else 
               {
                  m_nInitShArray[m_nInitShPosCount] = OrderTicket();
                  m_dInitShTriggerLevels[m_nInitShPosCount] = OrderOpenPrice() - (OrderStopLoss() - OrderOpenPrice()) / 2.0;
                  m_nInitShPosCount++;
               }               
            }     
         }    
      }
   }
   DisplayStatus();   
}

/////////////////////////////////////////////////////////////////////
// IsPositionStateChanged
/////////////////////////////////////////////////////////////////////
bool IsPositionStateChanged()
{
   if( (OrdersTotal() != m_nOrdersTotal) ||
       (OrdersHistoryTotal() != m_nOrdersHistoryTotal) ||
       (AccountMargin() != m_dAccountMargin) ||
       (AccountBalance() != m_dAccountBalance) ||
       (TimeCurrent() - m_dtTickTime > 5) 
     )
   {
      m_nOrdersTotal = OrdersTotal();
      m_nOrdersHistoryTotal = OrdersHistoryTotal();
      m_dAccountMargin = AccountMargin();
      m_dAccountBalance = AccountBalance();   
      m_dtTickTime = TimeCurrent();
      return (true);
   }
   return (false);
}

/////////////////////////////////////////////////////////////////////
// ValidateInitPositions
/////////////////////////////////////////////////////////////////////
void ValidateInitPositions(int nPosCount, int arrPosArray[], double arrTriggerLevel[], double dPrice, bool bIsGreater)
{
   int nLastError = 0;
   double dLots = 0.0;
   int nTicket = 0;
   for( int i = 0; i < nPosCount ; i++ )
   {
      if( OrderSelect(arrPosArray[i], SELECT_BY_TICKET) )
      {
         if( OrderCloseTime() == 0 )
         {
            nTicket = OrderTicket();
            if( bIsGreater && (dPrice > arrTriggerLevel[i]) )
            {
               dLots = GetNormalizedLotSize(OrderLots() * 0.70);
               ClosePosition(nTicket, dLots, dPrice, nLastError);
               updateSlOfPartialTicket(nTicket);
            }
            else if ( !bIsGreater && (dPrice < arrTriggerLevel[i]) )
            {
               dLots = GetNormalizedLotSize(OrderLots() * 0.70);
               ClosePosition(nTicket,dLots, dPrice, nLastError);            
               updateSlOfPartialTicket(nTicket);
            }
         }
      }
   }
}

/////////////////////////////////////////////////////////////////////
// ClosePosition
/////////////////////////////////////////////////////////////////////
bool ClosePosition(int nTicket, double dLots, double dPrice, int &nLastError)
{
   bool bClosed = false;
   int  nAttempts = 0;
   nLastError = 0;
   if( OrderSelect(nTicket, SELECT_BY_TICKET, MODE_TRADES) )
   {
      if( OrderCloseTime() == 0 )
      {
         while ( !bClosed && nAttempts < 10 )
         {
            nAttempts++;
            if( IsTradeContextBusy() ) Sleep(50);
            bClosed = OrderClose(nTicket, dLots, dPrice, SLIPPAGE * DIGIT_MULTIPLIER);
            if( bClosed )
            {
               m_bIsDirty = true;
               break;
            }
            else
            {
               int nError = GetLastError();
               if( nError > 1 )
               {
                  if( nError == ERR_TRADE_CONTEXT_BUSY ) 
                  {
                     nAttempts--;
                     Sleep(50);
                  }
                  else if( nAttempts >= 10 )
                  {
                     Alert( Symbol() + " : Could not close position " + nTicket + " at market. " + 
                            "Error " + nError + " [" + ErrorDescription(nError) + "].");            
                  }
                  RefreshRates();
               }
            }
         }    
      }
   }
   return (bClosed);
}

///////////////////////////////////////////////////////////////////////////////////////////////////
// GetNormalizedLotSize
// Returns the adjusted lot-size
///////////////////////////////////////////////////////////////////////////////////////////////////
double GetNormalizedLotSize(double dLots)
{
   double dMinLotSize = MarketInfo(Symbol(), MODE_MINLOT);
   double dLotSize = NormalizeDouble(dLots / MarketInfo(Symbol(), MODE_MINLOT), 0) * dMinLotSize;
   dLotSize = MathMin( MathMax(dLotSize, dMinLotSize) , MarketInfo(Symbol(), MODE_MAXLOT));
   return (dLotSize);
}

///////////////////////////////////////////////////////////////////////////////////////////////////
// GetPeriodEnumerator
// Returns the period enumerator from the string
///////////////////////////////////////////////////////////////////////////////////////////////////
int GetPeriodEnumerator(string sComment)
{
   int nPeriod = -1;
   if( sComment == "m1" || sComment == "M1" )
      nPeriod = PERIOD_M1;
   else if( sComment == "m5" || sComment == "M5" )
      nPeriod = PERIOD_M5;
   else if( sComment == "m15" || sComment == "M15" )
      nPeriod = PERIOD_M15;
   else if( sComment == "m30" || sComment == "m30" )
      nPeriod = PERIOD_M30;
   else if( sComment == "h1" || sComment == "H1" )
      nPeriod = PERIOD_H1;
   else if( sComment == "h4" || sComment == "H4" )
      nPeriod = PERIOD_H4;
   else if( sComment == "d" || sComment == "D" )
      nPeriod = PERIOD_D1;
   else if( sComment == "d1" || sComment == "D1" )
      nPeriod = PERIOD_D1;
   else if( sComment == "w" || sComment == "W" )
      nPeriod = PERIOD_W1;
   else if( sComment == "w1" || sComment == "W1" )
      nPeriod = PERIOD_W1;
   else if( sComment == "m" || sComment == "M" )
      nPeriod = PERIOD_MN1;
   else if( sComment == "mn" || sComment == "MN" )
      nPeriod = PERIOD_MN1;
   
   return (nPeriod);
}

/////////////////////////////////////////////////////////////////////
// ValidatePartialPositions
/////////////////////////////////////////////////////////////////////
void ValidatePartialPositions()
{
   int nLastError = 0;
   for( int i = 0; i < m_nPartPosCount ; i++ )
   {
      if( OrderSelect(m_nPartArray[i], SELECT_BY_TICKET) )
      {
         int nPeriod = m_nPartPeriod[i];
         double dEMA_Short = iMA(Symbol(), nPeriod, EMA_SHORTER_PERIOD, 0, MODE_EMA, PRICE_CLOSE, 1);
         double dEMA_Long  = iMA(Symbol(), nPeriod, EMA_LONGER_PERIOD, 0, MODE_EMA, PRICE_CLOSE, 1);
         if( (dEMA_Short > dEMA_Long) && (OrderType() == OP_SELL) )
            ClosePosition(OrderTicket(), OrderLots(), Ask, nLastError);            
         else if( (dEMA_Short < dEMA_Long) && (OrderType() == OP_BUY) ) 
            ClosePosition(OrderTicket(), OrderLots(), Bid, nLastError);
      }   
   }
}

/////////////////////////////////////////////////////////////////////
// GetPeriodFromComment
/////////////////////////////////////////////////////////////////////
int GetPeriodFromComment(string sComment)
{
   int nPeriod = -1;
   if( StringFind(sComment, "from") >= 0 )
   {
      int nPos = StringFind(sComment, "#");
      if( nPos > 0 )
      {
         int nOrgPos = StrToDouble(StringSubstr(sComment, nPos+1));
         if( OrderSelect(nOrgPos, SELECT_BY_TICKET, MODE_HISTORY) )
         {
            int nLastError = 0;
            string sPeriod = "-1";
            if (ReadProperty(m_sConfigFile, nOrgPos, sPeriod, nLastError) )
               nPeriod = StrToDouble(sPeriod);
         }      
      }
   }   
   return (nPeriod);
}

/////////////////////////////////////////////////////////////////////
// swapInt
/////////////////////////////////////////////////////////////////////
void swapInt(int &a, int &b)
{
   int temp = a;
   a = b;
   b = temp;
}

/////////////////////////////////////////////////////////////////////
// DisplayStatus
/////////////////////////////////////////////////////////////////////
void DisplayStatus()
{
   int nWindowLocation = getMt4CoOrdFromUserCoOrd();
   if( ObjectFind(TEXT_ID_LINE_1) == -1 )
   {
      ObjectCreate(TEXT_ID_LINE_1, OBJ_LABEL, 0, 0, 0);
      ObjectSet(TEXT_ID_LINE_1, OBJPROP_CORNER, nWindowLocation );
      ObjectSet(TEXT_ID_LINE_1, OBJPROP_XDISTANCE, 4);
      ObjectSet(TEXT_ID_LINE_1, OBJPROP_YDISTANCE, 20);
   }
   ObjectSetText(TEXT_ID_LINE_1, "New Positions : " + (m_nInitLnPosCount + m_nInitShPosCount), 8, "Arial", Red);

   if( ObjectFind(TEXT_ID_LINE_2) == -1 )
   {
      ObjectCreate(TEXT_ID_LINE_2, OBJ_LABEL, 0, 0, 0);
      ObjectSet(TEXT_ID_LINE_2, OBJPROP_CORNER, nWindowLocation );
      ObjectSet(TEXT_ID_LINE_2, OBJPROP_XDISTANCE, 4);
      ObjectSet(TEXT_ID_LINE_2, OBJPROP_YDISTANCE, 35);
   }
   ObjectSetText(TEXT_ID_LINE_2, "Partial Positions : " + (m_nPartPosCount), 8, "Arial", Red);

}

/////////////////////////////////////////////////////////////////////////////////////////
// getMt4CoOrdFromUserCoOrd
/////////////////////////////////////////////////////////////////////////////////////////
int getMt4CoOrdFromUserCoOrd()
{
   int nResult = 0;
   if( STATUS_WINDOW_LOCATION == 1 )
      nResult = 0;
   else if( STATUS_WINDOW_LOCATION == 2 )
      nResult = 1;
   else if( STATUS_WINDOW_LOCATION == 3 )
      nResult = 3;
   else if( STATUS_WINDOW_LOCATION == 4 )
      nResult = 2;
   return (nResult);
}

/////////////////////////////////////////////////////////////////////////////////////////
// GetPartialOrderTicket
/////////////////////////////////////////////////////////////////////////////////////////
int GetPartialOrderTicket(string sComment)
{
   int nTicket = 0;
   int nPos = StringFind(sComment, "#");
   if( nPos > 0 )
      nTicket = StrToDouble(StringSubstr(sComment, nPos+1));
   return (nTicket);
}

//////////////////////////////////////////////////////////////////////
// SaveProperty
//////////////////////////////////////////////////////////////////////
bool SaveProperty(string sFile, string sKey, string sValue, int &nError)
{
   bool bSaved = false;
   nError = 0;
   int hFile = FileOpen(sFile, FILE_CSV | FILE_WRITE | FILE_READ);
   if( hFile > 0 )
   {  
      bool bReplaced = false;
      int nIndex = 0;
      string sLines[1024];
      while ( !FileIsEnding(hFile) )
      {
         string sLine = FileReadString(hFile);
         if( StringFind(sLine, sKey + "=") == 0 )
         {
            bReplaced = true;
            sLine = sKey + "=" + sValue;         
         }
         sLines[nIndex] = sLine;
         nIndex++;
      }   
      FileClose(hFile);
      hFile = 0;
      
      if( !bReplaced )
      {
         sLines[nIndex] = sKey + "=" + sValue;
         nIndex++;
      }
      
      hFile = FileOpen(sFile, FILE_CSV | FILE_WRITE);
      if( hFile > 0 )
      {
         for( int i = 0; i < nIndex ; i++ )
            if( StringLen(sLines[i]) > 0) FileWrite(hFile, sLines[i]);
         FileClose(hFile);      
         hFile = 0;
         bSaved = true;
      }
      else
         nError = GetLastError();
   }
   else
      nError = GetLastError();
   if (hFile > 0) FileClose(hFile);
   return (bSaved);
}

//////////////////////////////////////////////////////////////////////
// ReadProperty
//////////////////////////////////////////////////////////////////////
bool ReadProperty(string sFile, string sKey, string &sValue, int &nError)
{
   bool bRead = false;
   nError = 0;
   int hFile = FileOpen(sFile, FILE_CSV | FILE_WRITE | FILE_READ);
   if( hFile > 0 )
   {  
      int nIndex = 0;
      while ( !FileIsEnding(hFile) )
      {
         string sLine = FileReadString(hFile);
         int nPos = StringFind(sLine, sKey + "=");
         if( nPos == 0 )
         {
            bRead = true;
            sValue = "";
            nPos = StringFind(sLine, "=") + 1;
            sValue = StringSubstr(sLine, nPos);
            break;
         }
         nIndex++;
      }   
      FileClose(hFile);
      hFile = 0;
   }
   else
      nError = GetLastError();
   return (bRead);
}

//////////////////////////////////////////////////////////////////////
// IsNewBar
//////////////////////////////////////////////////////////////////////
bool IsNewBar()
{
   if( Time[0] > m_dtBarTime )
   {
      m_dtBarTime = Time[0];
      return (true);
   }
   return (false);
}

//////////////////////////////////////////////////////////////////////
// updateSlOfPartialTicket
//////////////////////////////////////////////////////////////////////
void updateSlOfPartialTicket(int nOrgTicket)
{
   if( !OrderSelect(nOrgTicket, SELECT_BY_TICKET) ) return;
   if( OrderStopLoss() == 0.0 ) return;
   double dProfitPoints = 0.0, dOrgSLPoints = 0.0, dNewSL = 0.0;
   if( OrderType() == OP_BUY )
   {
      dOrgSLPoints = OrderOpenPrice() - OrderStopLoss();
      dProfitPoints = OrderClosePrice() - OrderOpenPrice();
      dNewSL = MathMin(OrderStopLoss() + dProfitPoints, OrderStopLoss() + dOrgSLPoints / 2.0);
   }
   else if( OrderType() == OP_SELL )
   {
      dOrgSLPoints = OrderStopLoss() - OrderOpenPrice();
      dProfitPoints = OrderOpenPrice() - OrderClosePrice();      
      dNewSL = MathMax(OrderStopLoss() - dProfitPoints, OrderStopLoss() - dOrgSLPoints / 2.0);
   }
   if( dProfitPoints <= 0.0 ) return;
   dNewSL = NormalizeDouble(dNewSL, Digits);
   string sComment = "from #" + DoubleToStr(nOrgTicket, 0);
   int nTicket = 0, nError = 0;
   for( int i = 0; i < OrdersTotal(); i++ )
   {
      if( !OrderSelect(i, SELECT_BY_POS) ) continue;
      if( OrderStopLoss() == dNewSL ) continue;
      if( StringFind(OrderComment(), sComment) >= 0 )
      {
         nTicket = OrderTicket();
         if( !ModifyOrder(nTicket, dNewSL, OrderTakeProfit(), nError) )
         {
            showErrorMessage("Could not modify S/L of Ticket " + DoubleToStr(nTicket, 0) + 
                             " to " + DoubleToStr(dNewSL, Digits) + ".", nError);
         }      
      }
   }
}

///////////////////////////////////////////////////////////////////////////////
// ModifyOrder
// Update the stop-loss and take-profit of the specified Order ticket.
///////////////////////////////////////////////////////////////////////////////
bool ModifyOrder(int nTicket, double dStopLoss, double dTakeProfit, int &nErrResult)
{
   bool bModified = false;
   nErrResult = 0;
   if( OrderSelect(nTicket, SELECT_BY_TICKET) )
   {
      if( (OrderStopLoss() == dStopLoss) && (OrderTakeProfit() == dTakeProfit) )
         return (true);
      int nAttemptsLeft = 20;
      while ((bModified == false) && (nAttemptsLeft > 0) )
      {
         nAttemptsLeft--;
         nErrResult = 0;
         bModified = OrderModify(nTicket, OrderOpenPrice(), dStopLoss, dTakeProfit, 0);
         if( !bModified )
         {
            nErrResult = GetLastError();
            if( nErrResult == ERR_TRADE_CONTEXT_BUSY || nErrResult == ERR_SERVER_BUSY ||
                nErrResult == ERR_TRADE_TIMEOUT || nErrResult == ERR_PRICE_CHANGED ||
                nErrResult == ERR_OFF_QUOTES || nErrResult == ERR_BROKER_BUSY || 
                nErrResult == ERR_REQUOTE )
            {
               Sleep(100);
            }
            else if( nErrResult <= 1 )
            {
               bModified = true;
               break;           
            }
            else
            {
               Print("ModifyOrder FAILED : Ticket(" + nTicket + ") " + 
                     "S/L(" + DoubleToStr(dStopLoss, Digits) + ") " +  "T/P(" + DoubleToStr(dTakeProfit, Digits) + ") ." + 
                     "Error " + nErrResult + " [" + ErrorDescription(nErrResult) + ".]");
               break;
            }
         }   
      }   
   }
   else
      nErrResult = ERR_INVALID_TICKET;   
   return (bModified);
}

///////////////////////////////////////////////////////////////////////////////
// showErrorMessage
///////////////////////////////////////////////////////////////////////////////
void showErrorMessage(string sMessage, int nError)
{
   Alert( WindowExpertName() + " [" + Symbol() + "] - " + sMessage + 
          " Error[" + nError + "] " + ErrorDescription(nError));
}

